//    Black Penguin - Game for Linux
//    Copyright (C) 1999 Holger Priebs, pengu@priebs.de
//
//    Holger Priebs
//    Hasselkamp 39
//    D - 24119 Kronshagen / Germany
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 


#include <qfile.h>
#include <stdlib.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qkeycode.h>
#include <qtimer.h>
#include <qmenubar.h>
#include "apple.xpm"
#include "apple_mask.xpm"
#include "orange.xpm"
#include "orange_mask.xpm"
#include "choco.xpm"
#include "bomb.xpm"
#include "bomb_mask.xpm"
#include "background.xpm"
#include "background2.xpm"
#include "background3.xpm"
#include "background4.xpm"
#include "pengu.xpm"
#include "mask_pengu.xpm"
#include "glas.xpm"
#include "glas2.xpm"
#include "glas_mask.xpm"
#include "gameover.xpm"
#include "window.xpm"
#include "window_mask.xpm"
#include "gotcha.xpm"
#include "gotcha_mask.xpm"
#include "minusone.xpm"
#include "pingucoin.xpm"
#include "pingucoin_mask.xpm"
#include "pingucoinsmall.xpm"
#include "lives.xpm"
#include "levelicon.xpm"
#include "nextlevel.xpm"
#include "chessred.xpm"
#include "chessblue.xpm"
#include "chessred_mask.xpm"
#include "youwin.xpm"

#include "BlackPenguinData.h"

#define Inherited QWidget

// macro taken from K&R

#define ABSDIFF(a, b) ((a)>(b) ? (a) - (b) : (b) - (a))

// calculating x-position and y-position from field number

#define XPOS(a) ( (a) % 9  )
#define YPOS(a) ( (a) / 9  )

// we define offsets for the different pixmaps in order to use the same coordinates for all

#define penguoffsetx 20
#define penguoffsety 60
#define goffsx 18
#define goffsy 32
#define foeoffsx 28
#define foeoffsy 38

// how many 'lives' we have at the beginning of each level

#define initial_lives 2

// global data - not quite the OOP approach, but it works :-)

int credit = 0; 	// stores how many points we've reached
int tempcredit = 0;	// temporary storage for the credits reached in the current level, comparison
			// with max credits of the current levels tells us if we've done the level completely
int GameStarted; 	// various flags for the game's current state
int foeactive;		// flag for starting / stopping foe's activities
int captured; 		// the evil window got you!
int lives; 		// how many 'lives' we still have
int level_done; 	// done one level, proceed to the next one (if any)
int round_over; 	// lose one 'life', but level's not yet done
int GameOver; 		// no more lives
int countdown; 		// prevent the messages from quickly disappearing by using a countdown variable, max 1000 !!!
int level; 		// current level

// how many credits we get for each kind of goodie

int whatcredit[maxgoodies + 1] = { 0, 1, 1, 4, 4, 10, 10, 0, 0, 3, 3 };

typedef int av[81];

// important: for those levels where two goodies per field have to be collected, the first goodie musT have an 
// uneven index, the second one must have the subsequent (even) index. Both goodies must have the same value as
// defined in whatcredit (s.a.)

// 0 = empty
// 1 = blue glass
// 2 = red glass
// 3 = apple
// 4 = orange
// 5 = pengucoin
// 6 = choco
// 7 = bomb
// 8 = bomb 2nd for shadowing it in levels with 2nd goodie av.
// 9 = blue chess
// 10 = red chess

// provide initial position on screen

av level_pattern[13] = {
{ 
	0,0,0,0,0,0,0,0,0,	// 0 = blue glasses
	0,0,1,1,1,1,1,1,0,	 
	0,1,1,1,1,1,1,0,0,
	0,1,1,1,1,1,0,0,0,
	0,1,1,1,1,0,0,0,0,
	0,1,1,1,0,0,0,0,0,
	0,1,1,0,0,0,0,0,0,
	0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 1 = apples	(and oranges )
	0,0,3,3,3,3,3,3,0,
	0,3,3,3,3,3,3,0,0,
	0,3,3,3,3,3,0,0,0,
	0,3,3,3,3,0,0,0,0,
	0,3,3,3,0,0,0,0,0,
	0,3,3,0,0,0,0,0,0,
	0,3,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{
	0,0,0,0,0,0,0,0,0,	// 2 = pengucoin
	0,0,5,5,5,5,5,5,0,
	0,5,5,5,5,5,5,0,0,
	0,5,5,5,5,5,0,0,0,
	0,5,5,5,5,0,0,0,0,
	0,5,5,5,0,0,0,0,0,
	0,5,5,0,0,0,0,0,0,
	0,5,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 3 = blue glasses with bombs in the middle
	0,0,1,1,1,1,1,1,0,	 
	0,1,7,7,7,1,1,0,0,
	0,1,7,7,1,1,0,0,0,
	0,1,7,1,1,0,0,0,0,
	0,1,1,1,0,0,0,0,0,
	0,1,1,0,0,0,0,0,0,
	0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 4 = one way throuh the mine field
	0,0,1,1,1,1,7,7,0,	 
	0,7,7,7,7,1,7,0,0,
	0,7,1,1,1,1,0,0,0,
	0,7,1,7,7,0,0,0,0,
	0,7,1,7,0,0,0,0,0,
	0,7,5,0,0,0,0,0,0,
	0,7,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 5 = one way throuh the mine field, vice versa
	0,0,7,7,7,7,7,7,0,	 
	0,1,7,1,1,1,5,0,0,
	0,1,7,1,7,7,0,0,0,
	0,1,7,1,7,0,0,0,0,
	0,1,1,1,0,0,0,0,0,
	0,7,7,0,0,0,0,0,0,
	0,7,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 6 = red glasses, use only with single goodie av.
	0,0,2,2,2,2,2,2,0,	 
	0,2,2,2,2,2,2,0,0,
	0,2,2,2,2,2,0,0,0,
	0,2,2,2,2,0,0,0,0,
	0,2,2,2,0,0,0,0,0,
	0,2,2,0,0,0,0,0,0,
	0,2,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0,0,0,0,0,0,0,0,0,	// 7 = oranges with bombs, use only with single goodie av.
	0,0,4,7,4,4,4,4,0,	 
	0,4,4,4,4,4,4,0,0,
	0,7,4,7,4,4,0,0,0,
	0,4,4,4,7,0,0,0,0,
	0,4,4,4,0,0,0,0,0,
	0,4,4,0,0,0,0,0,0,
	0,4,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
},
{ 
	0, 0, 0, 0, 0, 0, 0, 0, 0,	// 8 = chess
	0, 0, 9,10, 9,10, 9,10, 0,	 
	0, 9,10, 9,10, 9,10, 0, 0,
	0,10, 9,10, 9,10, 0, 0, 0,
	0, 9,10, 9,10, 0, 0, 0, 0,
	0,10, 9,10, 0, 0, 0, 0, 0,
	0, 9,10, 0, 0, 0, 0, 0, 0,
	0,10, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0
},
{ 
	0, 0, 0, 0, 0, 0, 0, 0, 0,	// 9 = chess 2nd foe av.
	0, 0, 9, 9, 9, 9, 9, 9, 0,	 
	0, 9, 9, 9, 9, 9, 9, 0, 0,
	0, 9, 9, 9, 9, 9, 0, 0, 0,
	0, 9, 9, 9, 9, 0, 0, 0, 0,
	0, 9, 9, 9, 0, 0, 0, 0, 0,
	0, 9, 9, 0, 0, 0, 0, 0, 0,
	0, 9, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0
},
{ 
	0, 0, 0, 0, 0, 0, 0, 0, 0,	// 10 = chess 2nd foe av.
	0, 0, 9, 9, 9, 9, 9, 7, 0,	 
	0, 9, 9, 9, 9, 9, 7, 0, 0,
	0, 9, 9, 9, 9, 7, 0, 0, 0,
	0, 9, 9, 9, 7, 0, 0, 0, 0,
	0, 9, 9, 7, 0, 0, 0, 0, 0,
	0, 9, 7, 0, 0, 0, 0, 0, 0,
	0, 7, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0
},
{ 
	0, 0, 0, 0, 0, 0, 0, 0, 0,	// 11 = mixed w/bombs
	0, 0, 1, 7, 7, 7, 7, 7, 0,	 
	0, 7, 2, 7, 3, 4, 5, 0, 0,
	0, 7, 6, 7, 6, 7, 0, 0, 0,
	0, 9,10, 1, 2, 0, 0, 0, 0,
	0, 3, 7, 7, 0, 0, 0, 0, 0,
	0, 4, 7, 0, 0, 0, 0, 0, 0,
	0, 5, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0
},
{ 
	0, 0, 0, 0, 0, 0, 0, 0, 0,	// 12 = bomb in the middle
	0, 0, 7, 7, 7, 7, 7, 7, 0,	 
	0, 1, 1, 1, 1, 7, 7, 0, 0,
	0, 7, 1, 7, 1, 7, 0, 0, 0,
	0, 7, 1, 1, 7, 0, 0, 0, 0,
	0, 7, 7, 7, 0, 0, 0, 0, 0,
	0, 7, 7, 0, 0, 0, 0, 0, 0,
	0, 7, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0
}
};

av available; // holds the appropriate part of level_pattern (see above)

// determines where foe can legally move
av legal = 
{ 
	0,0,0,0,0,0,0,0,0,	
	0,1,1,1,1,1,1,1,0,	
	0,1,1,1,1,1,1,0,0,
	0,1,1,1,1,1,0,0,0,
	0,1,1,1,1,0,0,0,0,
	0,1,1,1,0,0,0,0,0,
	0,1,1,0,0,0,0,0,0,
	0,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0
};


av havebeenthere; // foe stores already visited positions here, initialized by Foe::reset()

// we use a structure to enable assignments between the tables

struct screen_coord {
	int x;
	int y;
};

// we do not calculate coordinates and use a table instead due to the picture's perspective

struct screen_coord sc[81] = { 
{0,0}, {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0}, 
{0,0}, {329,65},  {367,100}, {403,133}, {441,169}, {487,207}, {531,249}, {580,290}, {0,0},
{0,0}, {293,100}, {331,133}, {369,169}, {412,207}, {453,249}, {497,290}, {0,0},     {0,0},
{0,0}, {258,133}, {295,169}, {333,207}, {373,249}, {415,290}, {0,0},     {0,0},     {0,0}, 
{0,0}, {220,169}, {254,207}, {296,249}, {336,290}, {0,0},     {0,0},     {0,0},     {0,0}, 
{0,0}, {175,207}, {214,249}, {256,290}, {0,0},     {0,0},     {0,0},     {0,0},     {0,0},
{0,0}, {133,249}, {171,290}, {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0}, 
{0,0}, {84,290},  {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0}, 
{0,0}, {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0},     {0,0} 
};	

// now a structure that holds the level specific data

struct level_data {
	int backgroundcolor; 	// determine the level's background color
	int extra_lives;	// does completion of previous level leed to extra lives, if so, how many
	int pattern;		// which pattern to use for the goodies, use bombs etc.
	int foeactive;		// there may be easy level for practicing, not threatened by the foe
	int credit_factor;	// multiplied with the goodies' values
	int lastlevel;		// is the respective level the last one available? 
	int foesmart;		// is the foe dumb or smart in the respective level?
	int foedelay;		// where does the foe start walking, useful values btw. 0 - 9 
	int timer;		// timer interval in msecs
	int credits;		// how much credits can we get in this level
	int second;		// if there's a second goodie to be collected
};

struct level_data ld[maxlevels] = {
// level 1
{ 	0, 	// blue background
	0, 	// extra lives
	0, 	// pattern 0 = blue glasses
	0, 	// foeactive
	10, 	// credit_factor
	0, 	// lastlevel
	0, 	// foesmart
	5, 	// foedelay
	1000, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	0	// second goodie av.,  0 = no
},
// level 2
{ 	0, 	// blue background
	1, 	// extra lives
	6,	// pattern 6 = red glasses
	1, 	// foeactive
	10, 	// credit_factor
	0, 	// lastlevel
	0, 	// foesmart
	5, 	// foedelay
	1000, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	0	// second goodie av.,  0 = no
},
// level 3
{ 	0, 	// blue background
	0, 	// extra lives
	3,	// pattern
	1, 	// foeactive
	10, 	// credit_factor
	0, 	// lastlevel
	1, 	// foesmart
	5, 	// foedelay
	1000, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	0	// second goodie av.,  0 = no
},
// level 4
{ 	0, 	// blue background
	0, 	// extra lives
	0, 	// pattern
	1, 	// foeactive
	10, 	// credit_factor
	0, 	// lastlevel
	1, 	// foesmart
	5, 	// foedelay
	1000, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	1	// second goodie av.,  0 = no
},
// level 5
{ 	0, 	// blue background
	1, 	// extra lives
	3, 	// pattern
	1, 	// foeactive
	10, 	// credit_factor
	0, 	// lastlevel
	1, 	// foesmart
	5, 	// foedelay
	800, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	1	// second goodie av.,  0 = no
},
// level 6
{ 	0, 	// blue background
	0,	// extra lives
	4, 	// pattern
	1, 	// foeactive
	100, 	// credit_factor
	0, 	// lastlevel
	1, 	// foesmart
	5, 	// foedelay
	600, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	0	// second goodie av.,  0 = no
},
// level 7
{ 	0, 	// blue background
	0, 	// extra lives
	5, 	// pattern
	1, 	// foeactive
	100, 	// credit_factor
	0, 	// lastlevel
	0, 	// foesmart
	5, 	// foedelay
	700, 	// timer
	0,	// do not fill in anything, calculated by calcualte_credits()
	1	// second goodie av.,  0 = no
},
// level 8
{ 	1, 	// green background
	1, 	// extra lives
	1, 	// pattern # 
	1, 	// foeactive
	20, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	8,	// foedelay
	500,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 9
{ 	1, 	// green background
	0, 	// extra lives
	1, 	// pattern #  
	1, 	// foeactive
	20, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	6,	// foedelay
	550,	// timer
	0,	// do not fill in
	1	// second goodie av.
},
// level 10
{ 	1, 	// green background
	2, 	// extra lives
	7, 	// pattern # 
	1, 	// foeactive
	20, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	8,	// foedelay
	600,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 11
{ 	3,	// white background
	0, 	// extra lives
	8, 	// pattern # (chess)
	1, 	// foeactive
	10, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	6,	// foedelay
	600,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 12
{ 	3,	// white background
	0, 	// extra lives
	9, 	// pattern # (chess 2nd f av.)
	1, 	// foeactive
	10, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	6,	// foedelay
	600,	// timer
	0,	// do not fill in
	1	// second goodie av.
},
// level 13
{ 	3,	// white background
	1, 	// extra lives
	10,	// pattern # (chess)
	1, 	// foeactive
	10, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	8,	// foedelay
	700,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 14
{ 	2,	// yellow background
	0, 	// extra lives
	11,	// pattern # (mixed)
	1, 	// foeactive
	10, 	// credit factor
	0,	// lastlevel 
	0,	// foesmart
	8,	// foedelay
	800,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 15
{ 	2,	// yellow background
	0, 	// extra lives
	12,	// pattern # (bomb in the middle)
	1, 	// foeactive
	10, 	// credit factor
	0,	// lastlevel 
	1,	// foesmart
	7,	// foedelay
	500,	// timer
	0,	// do not fill in
	0	// second goodie av.
},
// level 16
{ 	2, 	// yellow background
	0,	// extra lives
	2, 	// pattern # 2 (pengucoins w/ chocolate)
	1, 	// foeactive
	30, 	// credit factor 
	1,	// lastlevel
	0,	// foesmart
	5,	// foedelay
	1000,	// timer
	0,	// do not fill in
	1	// second goodie av.
},
};



// end global data


Foe::Foe() {

	// set up pixmap and mask
	m_foeBP = new QPixmap( window_xpm );
	m_foemaskBP = new QPixmap( window_mask_xpm ); 
	m_bm_foemaskBP = new QBitmap;
	*m_bm_foemaskBP = *m_foemaskBP;
	m_foeBP->setMask(*m_bm_foemaskBP);

	m_position = ld[level].foedelay; //leave the player some time to leave topmost place

	foeactive = 0; // do not start foe before game has been started
	for (int n=0; n<81; n++) {
		havebeenthere[n] = 0;
	}
	counter = 0; // how many times we've been called 
	
}

int Foe::current_pos() {
	return m_position;
}


void Foe::show( QPaintDevice* qpd ) {
	
	// show foe only on legal positions of the playing field
	if (legal[m_position]) bitBlt(qpd, sc[m_position].x - foeoffsx, 
		sc[m_position].y - foeoffsy, m_foeBP );

}

int Foe::legal_move(int position) {

	if (legal[position]) return 1; else return 0;
}

// this member function determines the foe's behaviour, called by timerevent()

void Foe::update( int penguposition ) {

	if ( !foeactive ) return;
	if( GameOver )return; 

	havebeenthere[m_position] = 1; // store where foe has already been
	
	// first steps towards the topmost field

	if (m_position < 10) {
		m_position++;
		return;
	}

	counter++;
	
	// now find out legal moves for the foe
	// we store the move itself for later processing
	// contents are { legal[0|1], move[m_position (-1|+1|-9|+9)], rating[-999..n]}
	int fourdir[3][4]= {{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0,0}};
	fourdir[1][0] = m_position + 1; // move right
	fourdir[1][1] = m_position - 1; // move left
	fourdir[1][2] = m_position - 9; // move up
	fourdir[1][3] = m_position + 9; // move down
	if (legal_move(fourdir[1][0])) fourdir[0][0] = 1; // move right
	if (legal_move(fourdir[1][1])) fourdir[0][1] = 1; // move left
	if (legal_move(fourdir[1][2])) fourdir[0][2] = 1; // move up
	if (legal_move(fourdir[1][3])) fourdir[0][3] = 1; // move down

	// now find out which one leads closest to the penguin

	int xpos = penguposition % 9;
	int ypos = penguposition / 9;
	int distance;
	int rdistance = 0;
	
	// leave the penguin a little chance, go left if it's right and vice versa, unless ld[level].foesmart set
	if ( !ld[level].foesmart ) {
		if (counter < (12 - ld[level].foedelay) ) {
			if ( xpos > ypos ) m_position += 9;
			if ( xpos < ypos ) m_position += 1;
			// if pengu's in the middle, walk right. Prevents foe from standing still
			if ( xpos == ypos) m_position += 1; 
			return;
		}
	}


	for (int bla=0; bla < 4; bla++) {
		// set illegal moves to negative value, we don't want them
		if (!(fourdir[0][bla])) fourdir[2][bla] = -999999;
		// highest priority: will capture penguin
		if (iscaptured( fourdir[1][bla], penguposition)) fourdir[2][bla] += 50000;
		// second highest priority: can capture penguin
		if (cancapture( fourdir[1][bla], penguposition)) fourdir[2][bla] += 30000; // 500
		// next level of priority: where is the penguin

		distance = ( ABSDIFF( XPOS( fourdir[1][bla] ), XPOS( penguposition ) ) ) +
				( ABSDIFF( YPOS( fourdir[1][bla] ), YPOS( penguposition ) ) );


		switch( distance ) {
		
			case 1: 	rdistance = 9000; break;
			case 2: 	rdistance = 8000; break;
			case 3: 	rdistance = 7000; break;
			case 4: 	rdistance = 6000; break;
			case 5: 	rdistance = 5000; break;
			case 6: 	rdistance = 4000; break;
			case 7: 	rdistance = 2000; break;
			case 8: 	rdistance = 1000; break;
			case 9: 	rdistance = 500; break;
			case 10:	rdistance = 200; break;
			case 11: 	rdistance = 100; break;
			case 12: 	rdistance = 0; break;
		}

		fourdir[2][bla] += rdistance;		
		rdistance = 0;

		// lowest priority: already visited
		if (!(havebeenthere[fourdir[1][bla]])) fourdir[2][bla] += 200;
			
	} //for
	
	// chose the foe's best move by fourdir[2][n]

	int ratings[2];

	if (fourdir[2][0] > fourdir[2][1]) ratings[0] = 0;
	else ratings[0] = 1;

	if (fourdir[2][2] > fourdir[2][3]) ratings[1] = 2;
	else ratings[1] = 3;

	if (fourdir[2][ratings[0]] > fourdir[2][ratings[1]]) m_position = fourdir[1][ratings[0]];
	else m_position = fourdir[1][ratings[1]]; 
	
}

void Foe::reset() {
	m_position = ld[level].foedelay ; //  leave the player some time to leave topmost place
	if ( ld[ level ].foeactive ) foeactive = 1; else foeactive = 0;
	counter = 0;
	for (int n=0; n<81; n++) {
		havebeenthere[n] = 0;
	}
}

int Foe::iscaptured(int fposition, int penguposition) {
	if (fposition == (penguposition)) return 1; // testhalber !!!
	return 0;

	// make the foe jump on poor penguin
	//if (captured) m_position = pinguposition;
}


int Foe::cancapture(int fposition, int penguposition) {
	if (fposition == (penguposition+1)) return 1; 
	if (fposition == (penguposition-1)) return 1; 
	if (fposition == (penguposition+9)) return 1; 
	if (fposition == (penguposition-9)) return 1; 
	return 0;
}		


void Foe::setposition(int there) {
	if (legal_move(there))m_position = there;
}

// goodies to collect from the playing field

Goodies::Goodies() {

	// set up pixmaps and masks for the collectable goodies
	m_goodiesBP[0] = new QPixmap( glas_xpm );
	m_goodiesmaskBP[0] = new QPixmap( glas_mask_xpm ); 
	m_bm_goodiesmaskBP[0] = new QBitmap;
	*m_bm_goodiesmaskBP[0] = *m_goodiesmaskBP[0];
	m_goodiesBP[0]->setMask(*m_bm_goodiesmaskBP[0]);

	m_goodiesBP[1] = new QPixmap( glas2_xpm );
	m_goodiesmaskBP[1] = new QPixmap( glas_mask_xpm ); 
	m_bm_goodiesmaskBP[1] = new QBitmap;
	*m_bm_goodiesmaskBP[1] = *m_goodiesmaskBP[1];
	m_goodiesBP[1]->setMask(*m_bm_goodiesmaskBP[1]);

	m_goodiesBP[2] = new QPixmap( apple_xpm );
	m_goodiesmaskBP[2] = new QPixmap( apple_mask_xpm ); 
	m_bm_goodiesmaskBP[2] = new QBitmap;
	*m_bm_goodiesmaskBP[2] = *m_goodiesmaskBP[2];
	m_goodiesBP[2]->setMask(*m_bm_goodiesmaskBP[2]);

	m_goodiesBP[3] = new QPixmap( orange_xpm );
	m_goodiesmaskBP[3] = new QPixmap( orange_mask_xpm ); 
	m_bm_goodiesmaskBP[3] = new QBitmap;
	*m_bm_goodiesmaskBP[3] = *m_goodiesmaskBP[3];
	m_goodiesBP[3]->setMask(*m_bm_goodiesmaskBP[3]);

	m_goodiesBP[4] = new QPixmap( pingucoin_xpm );
	m_goodiesmaskBP[4] = new QPixmap( pingucoin_mask_xpm ); 
	m_bm_goodiesmaskBP[4] = new QBitmap;
	*m_bm_goodiesmaskBP[4] = *m_goodiesmaskBP[4];
	m_goodiesBP[4]->setMask(*m_bm_goodiesmaskBP[4]);

	m_goodiesBP[5] = new QPixmap( choco_xpm );
	m_goodiesmaskBP[5] = new QPixmap( pingucoin_mask_xpm ); 
	m_bm_goodiesmaskBP[5] = new QBitmap;
	*m_bm_goodiesmaskBP[5] = *m_goodiesmaskBP[5];
	m_goodiesBP[5]->setMask(*m_bm_goodiesmaskBP[5]);

	m_goodiesBP[6] = new QPixmap( bomb_xpm );
	m_goodiesmaskBP[6] = new QPixmap( bomb_mask_xpm ); 
	m_bm_goodiesmaskBP[6] = new QBitmap;
	*m_bm_goodiesmaskBP[6] = *m_goodiesmaskBP[6];
	m_goodiesBP[6]->setMask(*m_bm_goodiesmaskBP[6]);

	m_goodiesBP[7] = new QPixmap( bomb_xpm );
	m_goodiesmaskBP[7] = new QPixmap( bomb_mask_xpm ); 
	m_bm_goodiesmaskBP[7] = new QBitmap;
	*m_bm_goodiesmaskBP[7] = *m_goodiesmaskBP[7];
	m_goodiesBP[7]->setMask(*m_bm_goodiesmaskBP[7]);

	m_goodiesBP[8] = new QPixmap( chessblue_xpm );
	m_goodiesmaskBP[8] = new QPixmap( chessred_mask_xpm ); 
	m_bm_goodiesmaskBP[8] = new QBitmap;
	*m_bm_goodiesmaskBP[8] = *m_goodiesmaskBP[8];
	m_goodiesBP[8]->setMask(*m_bm_goodiesmaskBP[8]);

	m_goodiesBP[9] = new QPixmap( chessred_xpm );
	m_goodiesmaskBP[9] = new QPixmap( chessred_mask_xpm ); 
	m_bm_goodiesmaskBP[9] = new QBitmap;
	*m_bm_goodiesmaskBP[9] = *m_goodiesmaskBP[9];
	m_goodiesBP[9]->setMask(*m_bm_goodiesmaskBP[9]);

}

// show the collectable goodies on the screen, don't show the ones already collected

void Goodies::show( QPaintDevice* qpd ) {

	for (int n=0; n<81; n++) {
		if (available[n] != 0) {
			bitBlt(qpd, 
				sc[n].x - goffsx, 
				sc[n].y - goffsy, 
				m_goodiesBP[available[n] - 1] );
		}
	};
}


// now the class for our little penguin

BP::BP() {

	// set up pixmap and mask
	m_penguBP = new QPixmap( pengu );
	m_pengumaskBP = new QPixmap(mask_pengu_xpm); 
	m_bm_pengumaskBP = new QBitmap;
	*m_bm_pengumaskBP = *m_pengumaskBP;
	m_penguBP->setMask(*m_bm_pengumaskBP);

	// provide initial position on screen
	m_position = 10;
}

void BP::show( QPaintDevice* qpd ) {
	// if pengu jumped on a step flgged as invalid
	// if jumped off or stepped on a bomb
	if (((sc[m_position].x == 0) && (sc[m_position].y == 0)) || (level_pattern[ ld[level].pattern ][m_position] == 7 ) ){
		GameStarted = 0;
		round_over = 1;
		foeactive = 0;
		if  (lives == 0) GameOver = 1;
	}
	if (!round_over)bitBlt(qpd, sc[m_position].x - penguoffsetx, 
		sc[m_position].y - penguoffsety, m_penguBP );

}

void BP::down() {

	m_position += 9;
	credit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	tempcredit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	if ( ld[level].second && available[m_position] % 2 != 0 ) available[m_position] +=1;
	else available[m_position] = 0; // delete entry for goodie on steps already visited
	if ( tempcredit == ld[level].credits ) {
		level_done = 1;
		foeactive = 0;
	}
	
}

void BP::up() {

	m_position -= 9;
	credit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	tempcredit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	if ( ld[level].second && available[m_position] % 2 != 0 ) available[m_position] +=1;
	else available[m_position] = 0; // delete entry for goodie on steps already visited
	if ( tempcredit == ld[level].credits ) {
		level_done = 1;
		foeactive = 0;
	}

}

void BP::right() {

	m_position += 1;
	credit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	tempcredit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	if ( ld[level].second && available[m_position] % 2 != 0 ) available[m_position] +=1;
	else available[m_position] = 0; // delete entry for goodie on steps already visited
	if ( tempcredit == ld[level].credits ) {
		level_done = 1;
		foeactive = 0;
	}

}

void BP::left() {

	m_position -= 1;
	credit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	tempcredit += whatcredit[ available[m_position] ] * ld[level].credit_factor;
	if ( ld[level].second && available[m_position] % 2 != 0 ) available[m_position] +=1;
	else available[m_position] = 0; // delete entry for goodie on steps already visited
	if ( tempcredit == ld[level].credits ) {
		level_done = 1;
		foeactive = 0;
	}

}

BlackPenguinData::BlackPenguinData
(
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name, 0 )
{
	// load a previously stored keyboard assignment if available, else use defaults
	if (!loadkeys()) {
		m_nKeyNW = Key_Left;
		m_nKeyNE = Key_Up;	
		m_nKeySW = Key_Down;
		m_nKeySE = Key_Right;
	}

	// calculate the values for max credits of each level so we need not hard code the values
	calculate_credits();

	// dummy kbd event
	m_keydummy = new QKeyEvent(Event_KeyPress,0,0,0);

	// usr has to start game first

	GameStarted = 0;

	// the game has just begun

	GameOver = 0;

	// handle keyboard events

	setFocusPolicy( StrongFocus );	

	// pointers to the penguin's and the goodies' classes and to the foe

	pbp = new BP;
	pg = new Goodies;
	pfoe = new Foe;


	// first create a common palette for all labels a.s.f.

	QColorGroup normal_all( QColor( QRgb(0) ), QColor( QRgb(16296080) ), QColor( QRgb(16310472) ), QColor( QRgb(8412240) ), QColor( QRgb(16287864) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
	QColorGroup disabled_all( QColor( QRgb(8421504) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(8421504) ), QColor( QRgb(10789024) ), QColor( QRgb(8421504) ), QColor( QRgb(12632256) ) );
	QColorGroup active_all( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(8421504) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
	QPalette palette_forall( normal_all, disabled_all, active_all );

	// create LCD counter for credits

	LCDcounter = new QLCDNumber( this, "LCDNumber_1" );
	LCDcounter->setGeometry( 540, 20, 100, 40 ); //20,100,100,30
	LCDcounter->setMinimumSize( 10, 10 );
	LCDcounter->setMaximumSize( 32767, 32767 );
	LCDcounter->setPalette( palette_forall );
	LCDcounter->setFrameStyle( 50 ) ; //( 33 );
	LCDcounter->setSmallDecimalPoint( FALSE );
	LCDcounter->setNumDigits( 5 );
	LCDcounter->setMode( QLCDNumber::DEC );
	LCDcounter->setSegmentStyle( QLCDNumber::Outline );

	// create label to explain the LCD

	QPixmap* creditpixmap = new QPixmap( pingucoinsmall_xpm );
	QLabel * creditlabel = new QLabel(this);
	creditlabel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	creditlabel->setGeometry( 505, 20, 30, 30 );
	creditlabel->setPalette( palette_forall );
	creditlabel->setPixmap( *creditpixmap );
	
	// create second LCD counter for # of lives

	LCDcounter2 = new QLCDNumber( this, "LCDNumber_2" );
	LCDcounter2->setGeometry( 540, 65, 100, 40 ); //20,100,100,30
	LCDcounter2->setMinimumSize( 10, 10 );
	LCDcounter2->setMaximumSize( 32767, 32767 );
	LCDcounter2->setPalette( palette_forall );
	LCDcounter2->setFrameStyle( 50 );
	LCDcounter2->setSmallDecimalPoint( FALSE );
	LCDcounter2->setNumDigits( 5 );
	LCDcounter2->setMode( QLCDNumber::DEC );
	LCDcounter2->setSegmentStyle( QLCDNumber::Outline );

	// create second label to explain the LCD

	QPixmap* creditpixmap2 = new QPixmap( lives_xpm );
	QLabel * creditlabel2 = new QLabel(this);
	creditlabel2->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	creditlabel2->setGeometry( 505, 65, 30, 30 );
	creditlabel2->setPalette( palette_forall );
	creditlabel2->setPixmap( *creditpixmap2 );

	// create third LCD counter for # of level

	LCDcounter3 = new QLCDNumber( this, "LCDNumber_2" );
	LCDcounter3->setGeometry( 540, 100 +10, 100, 40 ); //20,100,100,30
	LCDcounter3->setMinimumSize( 10, 10 );
	LCDcounter3->setMaximumSize( 32767, 32767 );
	LCDcounter3->setPalette( palette_forall );
	LCDcounter3->setFrameStyle( 50 );
	LCDcounter3->setSmallDecimalPoint( FALSE );
	LCDcounter3->setNumDigits( 5 );
	LCDcounter3->setMode( QLCDNumber::DEC );
	LCDcounter3->setSegmentStyle( QLCDNumber::Outline );

	// create third label to explain the LCD

	QPixmap* creditpixmap3 = new QPixmap( levelicon_xpm );
	QLabel * creditlabel3 = new QLabel(this);
	creditlabel3->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	creditlabel3->setGeometry( 505, 110, 30, 30 );
	creditlabel3->setPalette( palette_forall );
	creditlabel3->setPixmap( *creditpixmap3 );

	// initialize background

	m_invisible_bg[0] = new QPixmap( background_xpm );  // blue stairs
	m_invisible_bg[1] = new QPixmap( background2_xpm ); // green stairs
	m_invisible_bg[2] = new QPixmap( background3_xpm ); // yellow stairs
	m_invisible_bg[3] = new QPixmap( background4_xpm ); // white stairs
	m_visible_bg = new QPixmap( background_xpm );       

	// pixmap 'got you'

	m_gotcha = new QPixmap( gotcha_xpm );
	m_gotchamask = new QPixmap( gotcha_mask_xpm ); 
	m_gotchamaskBM = new QBitmap;
	*m_gotchamaskBM = *m_gotchamask;
	m_gotcha->setMask(*m_gotchamaskBM);

	// pixmap you lose one life

	m_minusone = new QPixmap( minusone_xpm );

	// initialize 'game over' pixmap
	m_gameover = new QPixmap( gameover_xpm );

	// initialize 'prepare for next level' pixmap

	m_nextlevel = new QPixmap( nextlevel_xpm );

	// initialize 'you win' pixmap
	
	m_youwin = new QPixmap( youwin_xpm );

	// timer for foe actions

	ptimer = new QTimer(this);
	connect( ptimer, SIGNAL(timeout()), this, SLOT(timerevent()) );

	resize( 665,390 );
	setMinimumSize( 665, 390 );
	setMaximumSize( 665, 390 );

// --------  moved by Holger Priebs 21 Nov 1999
// added by Olaf Hartig 11 Nov 1999
//	m_nKeyNW = Key_Left;
//	m_nKeyNE = Key_Up;	
//	m_nKeySW = Key_Down;
//	m_nKeySE = Key_Right;
// --------
	
}

void BlackPenguinData::calculate_credits() {

	for (int i = 0; i < maxlevels; i++) {		// i stands for the level
		for (int n = 0;  n < 81; n++) {
			ld[i].credits +=  ( whatcredit[ level_pattern[ ld[i].pattern ][n] ] * ld[i].credit_factor);
		}
		
		if ( ld[i].second ) ld[i].credits *= 2; // double if there's a second set of goodies to collect
	}

}

void BlackPenguinData::timerevent() {

	// update the evil foe's poisition according to pengu's current position
	pfoe->update(pbp->m_position); 
	LCDcounter2->display(lives);
	keyPressEvent(m_keydummy);
}

void BlackPenguinData::paintEvent( QPaintEvent * )
{
	// we do nothing but blitting the elsewhere painted picture to the 
	// physical screen
	bitBlt(this, 0, 0, m_visible_bg );
}

int BlackPenguinData::savekeys() {

	char filename[255];
	strcpy( filename, getenv("HOME"));
	strcat( filename, "/.blackpenguinkeys");
	QFile f(filename);
	if ( f.open( IO_WriteOnly ) ) {
		// writeBlock() expects char*. Cast looks ugly, but it works
		f.writeBlock((char*)&m_nKeyNW, sizeof(int));
		f.writeBlock((char*)&m_nKeySW, sizeof(int));
		f.writeBlock((char*)&m_nKeyNE, sizeof(int));
		f.writeBlock((char*)&m_nKeySE, sizeof(int));
		f.close();
		return 0;
	}
	return 1;
}

int BlackPenguinData::loadkeys() {

	char filename[255];
	strcpy( filename, getenv("HOME"));
	strcat( filename, "/.blackpenguinkeys");
	QFile f(filename);
	if ( f.open( IO_ReadOnly ) ) {
		// writeBlock() expects char*. Cast looks ugly, but it works
		f.readBlock((char*)&m_nKeyNW, sizeof(int));
		f.readBlock((char*)&m_nKeySW, sizeof(int));
		f.readBlock((char*)&m_nKeyNE, sizeof(int));
		f.readBlock((char*)&m_nKeySE, sizeof(int));
		f.close();
		return TRUE;
	}
	return FALSE;
}

int BlackPenguinData::savegame() {

	char filename[255];
	strcpy( filename, getenv("HOME"));
	strcat( filename, "/.blackpenguin");
	QFile f(filename);
	if (GameOver) return 1; // you cannot save a game that's over
	if ( f.open( IO_WriteOnly ) ) {
		// writeBlock() expects char*. Cast looks ugly, but it works
		f.writeBlock((char*)&credit, sizeof(int));
		f.writeBlock((char*)&lives, sizeof(int));
		f.writeBlock((char*)&level, sizeof(int));
		f.close();
		return 0;
	}
	return 2;
}

int BlackPenguinData::loadgame() {

	char filename[255];
	strcpy( filename, getenv("HOME"));
	strcat( filename, "/.blackpenguin");
	QFile f(filename);
	if ( f.open( IO_ReadOnly ) ) {
		// readBlock() expects char*. Cast looks ugly, but it works
		f.readBlock((char*)&credit, sizeof(int));
		f.readBlock((char*)&lives, sizeof(int));
		f.readBlock((char*)&level, sizeof(int));
		f.close();
		// we did not store some variables which are essential for game control
		GameOver = 0;
		level_done = 0;
		// now behave as if we had been in a game before
		resetround();
		return 0;
	}
	return 1;
}

void BlackPenguinData::increaselevel() {

	if ( ld[level].lastlevel ) return;
	level_done = 0;
	tempcredit = 0;
	level++;
	// show # of level on the screen (internal level no + 1)
	LCDcounter3->display(level+1);
	// additonal 'lives' for completion of previous level
	lives += ld[level].extra_lives;
	// the round is not yet over
	round_over = 0;
	// we haven't done any levels yet
	level_done = 0;
	LCDcounter->display(credit);
	// display lives
	LCDcounter2->display(lives);
	// first delete the background by overwriting it with the new one			
	bitBlt( m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] ); 
	// then put the goodies on all the places
	available = level_pattern[ld[level].pattern];
	// put the little penguin on the top field
	pbp->m_position = 10;
	// blit the goodies on the 'visible' screen
	pg->show( m_visible_bg );
	// finally blit the little penguin on the pixmap
	pbp->show( m_visible_bg );
	// blit the whole thing on the physical screen
	bitBlt(this, 0, 0, m_visible_bg );
	// start the timer
	ptimer->start( ld[level].timer, FALSE);
	// reset the foe
	pfoe->reset();
	// we're still free
	captured = 0;
	// countdown for messages
	countdown = (1000 / ld[level].timer) * 3 ;

}

// --------  ((Zeile 1146))
// added by Olaf Hartig 11 Nov 1999
void BlackPenguinData::newKeys(int nKeyNW, int nKeyNE, int nKeySW, int nKeySE)
{
	m_nKeyNW = nKeyNW;
	m_nKeyNE = nKeyNE;
	m_nKeySW = nKeySW;
	m_nKeySE = nKeySE;	
// added by Holger Priebs 22 Nov 1999
	savekeys();
}
// --------
void BlackPenguinData::reset() {
	// reset level to 0 aka first level
	level = 0;
	// show # of level on the screen (internal level no + 1)
	LCDcounter3->display(level+1);
	// in the beginning we got as many lives as defined in initial_lives
	lives = initial_lives;
	// the round is not yet over
	round_over = 0;
	// we haven't done any levels yet
	level_done = 0;
	// game's not over
	GameOver = 0;
	// reset counter
	credit = 0;
	LCDcounter->display(credit);
	// display lives
	LCDcounter2->display(lives);
	// game has been started
	GameStarted = 1;
	// first delete the background by overwriting it with the original			
	bitBlt( m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] ); 
	// then put the goodies on all the places
	available = level_pattern[ld[level].pattern];
	// put the little penguin on the top field
	pbp->m_position = 10;
	// blit the goodies on the 'visible' screen
	pg->show( m_visible_bg );
	// finally blit the little penguin on the pixmap
	pbp->show( m_visible_bg );
	// blit the whole thing on the physical screen
	bitBlt(this, 0, 0, m_visible_bg );
	// start the timer
	ptimer->start( ld[level].timer , FALSE);
	// reset the foe
	pfoe->reset();
	// we're still free
	captured = 0;
	// countdown for messages
	countdown = (1000 / ld[level].timer) * 3 ;
	// reset the temporary credit counter to zero 
	tempcredit = 0;

}


void BlackPenguinData::resetround() {

	// variable tempcredit used to find out when the level is done
	tempcredit = 0;
	// the round is not yet over
	round_over = 0;
	// was it pengu's last 'life' ?
	if (lives==0) GameOver = 1;
	LCDcounter->display(credit);
	// display lives
	LCDcounter2->display(lives);
	// nothing left to do if game is over
	// show # of level on the screen (internal level no + 1)
	LCDcounter3->display(level+1);
	if ( GameOver ) return;
	// first delete the background by overwriting it with the original			
	bitBlt( m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor] ); 
	// then put the goodies on all the places
	available = level_pattern[ld[level].pattern];
	// put the little penguin on the top field
	pbp->m_position = 10;
	// blit the goodies on the 'visible' screen
	pg->show( m_visible_bg );
	// finally blit the little penguin on the pixmap
	pbp->show( m_visible_bg );
	// blit the whole thing on the physical screen
	bitBlt(this, 0, 0, m_visible_bg );
	// start the timer
	ptimer->start( ld[level].timer , FALSE);
	// reset the foe
	pfoe->reset();
	// we're still free
	captured = 0;
	// countdown for messages
	countdown = (1000 / ld[level].timer) * 3 ;
	//
	GameStarted = 1;

}


void BlackPenguinData::keyPressEvent( QKeyEvent* e )
{

	// have the foe evaluate if he's captured the pengu, but don't capture
	// the penguin when walking towards the playing field on illegal positions
	if (  (pfoe->iscaptured(pfoe->current_pos(), pbp->m_position)) &&  
		(pfoe->legal_move(pfoe->current_pos()))) {
		captured = 1;
		// stop moving
		foeactive = 0; 
		// jump on poor pengu
		pfoe->setposition(pbp->m_position);
	}

	// has to be before anything else: return if game's over	

	if ( GameOver ) {
			e->ignore();
			
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor  ] );
			pg->show( m_visible_bg ); // copy goodies to bg2
			pbp->show( m_visible_bg ); // show the penguin
			pfoe->show( m_visible_bg ); // copy the evil foe to bm2
			bitBlt(  m_visible_bg , 190, 140, m_gameover ); 
			bitBlt(this, 0, 0, m_visible_bg ); // blit to physical screen
			return;
	}

	// if we're captured by the evil window we can't move anymore
	if ( captured ) {
			e->ignore();

			// countdown to keep the message on the screen for a while
			if ( countdown == 0) {
				lives--;
				if ( lives == 0 ) {
					GameOver = 1;
					return;
				}
				resetround();
				return;
			}
			countdown--;

			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pg->show( m_visible_bg ); // copy goodies to bg2
			pbp->show( m_visible_bg ); 
			// make the foe jump on poor penguin
			pfoe->show( m_visible_bg ); // copy the evil foe to bm2
			bitBlt(  m_visible_bg , 190, 140, m_gotcha ); 
			bitBlt(this, 0, 0, m_visible_bg ); // blit to physical screen
			return;
	}

	if ( round_over ) {
			e->ignore();
			// countdown to keep the message on the screen for a while
			if ( countdown == 0) {
				lives--;
				round_over = 0;
				resetround();
				return;
			}
			countdown--;
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pg->show( m_visible_bg ); // copy goodies to bg2
			pfoe->show( m_visible_bg ); // copy the evil foe to bm2
			bitBlt(  m_visible_bg , 190, 140, m_minusone ); 
			bitBlt(this, 0, 0, m_visible_bg ); // blit to physical screen
			return;
	}

	if ( level_done ) {

			e->ignore();
			foeactive = 0;
			// countdown to keep the message on the screen for a while
			if ( countdown == 0) {
				increaselevel();
				return;
			}
			countdown--;
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pg->show( m_visible_bg ); // copy goodies to bg2
			pfoe->show( m_visible_bg ); // copy the evil foe to bm2
			
			if ( !ld[level].lastlevel )
			bitBlt(  m_visible_bg , 190, 140, m_nextlevel ); 
			else 
			{  
 				bitBlt(  m_visible_bg , 190, 140, m_youwin ); 
			}
			bitBlt(this, 0, 0, m_visible_bg ); // blit to physical screen
			return;

	}

// -------- 
// modified by Olaf Hartig 11 Nov 1999
//	switch( e->key() ) {

//		case Key_Down:
//			if (!GameStarted) break;

	if ((GameStarted)&&(e->key() ==	m_nKeySW))
	{
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pbp->down(); // change state of penguin
			pg->show( m_visible_bg ); // change & copy goodies to bg2
			pbp->show( m_visible_bg ); // actually copy penguin, i.e.
			pfoe->show( m_visible_bg ); // copy the evil foe to bm2
			// change and copy penguin in two steps to ensure it's always on top
			bitBlt(this, 0, 0, m_visible_bg ); // blit to physical screen
			LCDcounter->display(credit);
//			break;
	}
	
//		case Key_Up:
//			if (!GameStarted) break;
	else if ((GameStarted)&&(e->key() == m_nKeyNE))
	{
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pbp->up();
			pg->show( m_visible_bg );
			pbp->show( m_visible_bg );
			pfoe->show( m_visible_bg );
			bitBlt(this, 0, 0, m_visible_bg );
			LCDcounter->display(credit);
//			break;
	}

//		case Key_Right:
//			if (!GameStarted) break;
	else if ((GameStarted)&&(e->key() == m_nKeySE))
	{
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pbp->right();
			pg->show( m_visible_bg );
			pbp->show( m_visible_bg );
			pfoe->show( m_visible_bg );
			bitBlt(this, 0, 0, m_visible_bg );
			LCDcounter->display(credit);
//			break;
	}

//		case Key_Left:
//			if (!GameStarted) break;
	else if ((GameStarted)&&(e->key() == m_nKeyNW))
	{
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pbp->left();
			pg->show( m_visible_bg );
			pbp->show( m_visible_bg );
			pfoe->show( m_visible_bg );
			bitBlt(this, 0, 0, m_visible_bg );
			LCDcounter->display(credit);
//			break;
	}

//		default:
	else
	{
			// keys not relevant for the application are handled here, i.e.
			// ignored. We use a dummy key event to handle all screen updates
			// (incl. those triggered by the timer) in one function
			bitBlt(m_visible_bg, 0, 0, m_invisible_bg[ ld[level].backgroundcolor ] );
			pg->show( m_visible_bg );
			if (GameStarted)pbp->show( m_visible_bg );
			pfoe->show( m_visible_bg );
			bitBlt(this, 0, 0, m_visible_bg );
			LCDcounter->display(credit); // ununcommented H.Priebs 16 Nov 99
			e->ignore();
			return;
	}
//	}
// --------

}


BlackPenguinData::~BlackPenguinData()
{
}

